// 
// SVEVolatilityBand is Copyright (C) 2012, Sylvain Vervoort <stocata.org>.
// stocata.org reserves the right to modify or overwrite this NinjaScript component with each release.
// Release V1.0 March, 2012.

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// The SVEVolatilityBand uses an upper and lower band over price bars. 
	/// Price moves mostly withing this band.
	/// </summary>
	[Description("The SVEVolatilityBand uses an upper and lower volatility band over price bars")]
	public class SVEVolatilityBand : Indicator
	{
		#region Variables
		private int			average				=	8;		// Band average
		private int 		volperiod			=	13;		// Volatility Summing period
		private double 		devfact				=	3.55;	// Deviation factor
		private double		lowbandadjust		=	0.9;	// Low band adjustment factor
		
		private double 		devhigh, devlow;
		private DataSeries	typical;
		private DataSeries	deviation;
		private DataSeries	medianaverage;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator, called once before any bar is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(Color.Magenta, PlotStyle.Line, "Upper"));
			Add(new Plot(Color.Magenta, PlotStyle.Line, "Lower"));
			Add(new Plot(Color.Magenta, PlotStyle.Line, "Median"));
			
			Plots[0].Pen.Width 		= 2;
			Plots[0].Pen.DashStyle 	= DashStyle.Dot;
			Plots[1].Pen.Width 		= 2;
			Plots[1].Pen.DashStyle 	= DashStyle.Dot;
			Plots[2].Pen.Width 		= 1;
			Plots[2].Pen.DashStyle 	= DashStyle.Solid;
			
			typical			= new DataSeries(this);
			deviation		= new DataSeries(this);			
			medianaverage	= new DataSeries(this);		
			
			Overlay				= true;
			PaintPriceMarkers 	= false;
		}
		
		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			if (CurrentBar < 1) return;  // need 2 bars to start
			
			// basic volatility in new "typical" data series
			if (Typical[0] >= Typical[1]) typical[0] = Typical[0] - Low[1];
			else typical[0]	= Typical[1] - Low[0];
			
			// basic deviation based on "typical" over a period and a factor
			deviation.Set(SUM(typical,volperiod)[0] / volperiod * devfact);
			
			// the average deviation high and low band
			devhigh	= EMA(deviation,average)[0];
			devlow	= EMA(deviation,average)[0] * lowbandadjust;
			
			// the middle average reference
			medianaverage.Set(EMA(Typical,average)[0]);
			
			// creation of the upper and lower band
			if (CurrentBar > average*3+2*volperiod) // show only after it is stable
			{
			Upper.Set(EMA(medianaverage,average)[0] + devhigh);
			Lower.Set(EMA(medianaverage,average)[0] - devlow);
			Median.Set(SMA(medianaverage,average)[0]);
			}
		}
			
		#region Properties
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Upper
		{
			get { return Values[0]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Lower
		{
			get { return Values[1]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Median
		{
			get { return Values[2]; }
		}
		
		/// <summary>
		/// </summary>
		[Description("The Volatility Band average DEF=8")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayName("1. Band Average")]
		public int _Vol_Band_Average
		{
			get { return average; }
			set { average = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("The Volatility Look Back Summing Period DEF=13")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayName("2. Summing Period")]
		public int _Vol_Summing_Period
		{
			get { return volperiod; }
			set { volperiod = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("The Deviation Factor DEF=3.55")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayName("3. Deviation Factor")]
		public double Deviaton_Factor
		{
			get { return devfact; }
			set { devfact = Math.Max(0.1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Low band Adjustment DEF=0.9")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayName("4. Lower Band Adjust")]
		public double Low_Band_Adjust
		{
			get { return lowbandadjust; }
			set { lowbandadjust = Math.Max(0.01, value); }
		}
		
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SVEVolatilityBand[] cacheSVEVolatilityBand = null;

        private static SVEVolatilityBand checkSVEVolatilityBand = new SVEVolatilityBand();

        /// <summary>
        /// The SVEVolatilityBand uses an upper and lower volatility band over price bars
        /// </summary>
        /// <returns></returns>
        public SVEVolatilityBand SVEVolatilityBand(int _Vol_Band_Average, int _Vol_Summing_Period, double deviaton_Factor, double low_Band_Adjust)
        {
            return SVEVolatilityBand(Input, _Vol_Band_Average, _Vol_Summing_Period, deviaton_Factor, low_Band_Adjust);
        }

        /// <summary>
        /// The SVEVolatilityBand uses an upper and lower volatility band over price bars
        /// </summary>
        /// <returns></returns>
        public SVEVolatilityBand SVEVolatilityBand(Data.IDataSeries input, int _Vol_Band_Average, int _Vol_Summing_Period, double deviaton_Factor, double low_Band_Adjust)
        {
            if (cacheSVEVolatilityBand != null)
                for (int idx = 0; idx < cacheSVEVolatilityBand.Length; idx++)
                    if (cacheSVEVolatilityBand[idx]._Vol_Band_Average == _Vol_Band_Average && cacheSVEVolatilityBand[idx]._Vol_Summing_Period == _Vol_Summing_Period && Math.Abs(cacheSVEVolatilityBand[idx].Deviaton_Factor - deviaton_Factor) <= double.Epsilon && Math.Abs(cacheSVEVolatilityBand[idx].Low_Band_Adjust - low_Band_Adjust) <= double.Epsilon && cacheSVEVolatilityBand[idx].EqualsInput(input))
                        return cacheSVEVolatilityBand[idx];

            lock (checkSVEVolatilityBand)
            {
                checkSVEVolatilityBand._Vol_Band_Average = _Vol_Band_Average;
                _Vol_Band_Average = checkSVEVolatilityBand._Vol_Band_Average;
                checkSVEVolatilityBand._Vol_Summing_Period = _Vol_Summing_Period;
                _Vol_Summing_Period = checkSVEVolatilityBand._Vol_Summing_Period;
                checkSVEVolatilityBand.Deviaton_Factor = deviaton_Factor;
                deviaton_Factor = checkSVEVolatilityBand.Deviaton_Factor;
                checkSVEVolatilityBand.Low_Band_Adjust = low_Band_Adjust;
                low_Band_Adjust = checkSVEVolatilityBand.Low_Band_Adjust;

                if (cacheSVEVolatilityBand != null)
                    for (int idx = 0; idx < cacheSVEVolatilityBand.Length; idx++)
                        if (cacheSVEVolatilityBand[idx]._Vol_Band_Average == _Vol_Band_Average && cacheSVEVolatilityBand[idx]._Vol_Summing_Period == _Vol_Summing_Period && Math.Abs(cacheSVEVolatilityBand[idx].Deviaton_Factor - deviaton_Factor) <= double.Epsilon && Math.Abs(cacheSVEVolatilityBand[idx].Low_Band_Adjust - low_Band_Adjust) <= double.Epsilon && cacheSVEVolatilityBand[idx].EqualsInput(input))
                            return cacheSVEVolatilityBand[idx];

                SVEVolatilityBand indicator = new SVEVolatilityBand();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator._Vol_Band_Average = _Vol_Band_Average;
                indicator._Vol_Summing_Period = _Vol_Summing_Period;
                indicator.Deviaton_Factor = deviaton_Factor;
                indicator.Low_Band_Adjust = low_Band_Adjust;
                Indicators.Add(indicator);
                indicator.SetUp();

                SVEVolatilityBand[] tmp = new SVEVolatilityBand[cacheSVEVolatilityBand == null ? 1 : cacheSVEVolatilityBand.Length + 1];
                if (cacheSVEVolatilityBand != null)
                    cacheSVEVolatilityBand.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSVEVolatilityBand = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The SVEVolatilityBand uses an upper and lower volatility band over price bars
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SVEVolatilityBand SVEVolatilityBand(int _Vol_Band_Average, int _Vol_Summing_Period, double deviaton_Factor, double low_Band_Adjust)
        {
            return _indicator.SVEVolatilityBand(Input, _Vol_Band_Average, _Vol_Summing_Period, deviaton_Factor, low_Band_Adjust);
        }

        /// <summary>
        /// The SVEVolatilityBand uses an upper and lower volatility band over price bars
        /// </summary>
        /// <returns></returns>
        public Indicator.SVEVolatilityBand SVEVolatilityBand(Data.IDataSeries input, int _Vol_Band_Average, int _Vol_Summing_Period, double deviaton_Factor, double low_Band_Adjust)
        {
            return _indicator.SVEVolatilityBand(input, _Vol_Band_Average, _Vol_Summing_Period, deviaton_Factor, low_Band_Adjust);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The SVEVolatilityBand uses an upper and lower volatility band over price bars
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SVEVolatilityBand SVEVolatilityBand(int _Vol_Band_Average, int _Vol_Summing_Period, double deviaton_Factor, double low_Band_Adjust)
        {
            return _indicator.SVEVolatilityBand(Input, _Vol_Band_Average, _Vol_Summing_Period, deviaton_Factor, low_Band_Adjust);
        }

        /// <summary>
        /// The SVEVolatilityBand uses an upper and lower volatility band over price bars
        /// </summary>
        /// <returns></returns>
        public Indicator.SVEVolatilityBand SVEVolatilityBand(Data.IDataSeries input, int _Vol_Band_Average, int _Vol_Summing_Period, double deviaton_Factor, double low_Band_Adjust)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SVEVolatilityBand(input, _Vol_Band_Average, _Vol_Summing_Period, deviaton_Factor, low_Band_Adjust);
        }
    }
}
#endregion
